import logging
import traceback
from typing import List, Optional

from openlineage.airflow.extractors.base import TaskMetadata
from google.api_core.exceptions import NotFound

# TODO submit bug fix to OL for:
from openlineage.airflow.extractors.bigquery_extractor import BigQueryExtractor
from openlineage.airflow.utils import get_job_name
from openlineage.client.facet import SqlJobFacet
from openlineage.common.provider.bigquery import (
    BigQueryDatasetsProvider,
    BigQueryErrorRunFacet,
    BigQueryFacets,
)

log = logging.getLogger(__name__)


class AlvinBigQueryBaseExtractor(BigQueryExtractor):
    """
    This class contains common logic to handle extraction of BigQuery lineage
    for both Airflow 1 and 2.

    https://cloud.google.com/bigquery/docs/information-schema-jobs
    """

    def __init__(self, operator, *args, **kwargs):
        super().__init__(operator, *args, **kwargs)

    @classmethod
    def get_operator_classnames(cls) -> List[str]:
        raise NotImplementedError

    def extract_on_complete(self, task_instance) -> Optional[TaskMetadata]:
        log.debug(f"extract_on_complete({task_instance})")
        context = self.parse_sql_context()

        try:
            self.bigquery_job_id = self._get_xcom_bigquery_job_id(task_instance)

            if self.bigquery_job_id is None:
                raise Exception(
                    "Xcom could not resolve BigQuery job id." + "Job may have failed."
                )
        except Exception as e:
            log.error(
                f"Cannot retrieve job details from BigQuery.Client. {e}", exc_info=True
            )
            return TaskMetadata(
                name=get_job_name(task=self.operator),
                run_facets={
                    "bigQuery_error": BigQueryErrorRunFacet(
                        clientError=f"{e}: {traceback.format_exc()}",
                        parserError=context.parser_error,
                    )
                },
            )

        # BEGIN bugfix to OL
        log.info(f"Operator: {self.operator}")
        if self.operator.hook:
            bq_client = self.operator.hook.get_client(
                project_id=self.operator.hook.project_id
            )
            log.info(f"Using this bq_client: {bq_client}")

            stats = BigQueryDatasetsProvider(bq_client).get_facets(self.bigquery_job_id)
            # END bugfix to OL
            inputs = stats.inputs
            output = stats.output
            run_facets = stats.run_facets

            facets = {
                    # BEGIN custom Alvin stuff
                    "alvin": {
                        "project_id": self.operator.hook.project_id
                        if self.operator.hook
                        else None,
                        "job_id": self.bigquery_job_id,
                    },
                    # END custom Alvin stuff
                    "sql": SqlJobFacet(context.sql),
                }
            log.info(f"Facets: {facets}")
            return TaskMetadata(
                name=get_job_name(task=self.operator),
                inputs=[ds.to_openlineage_dataset() for ds in inputs],
                outputs=[output.to_openlineage_dataset()] if output else [],
                run_facets=run_facets,
                job_facets=facets
            )
        else:
            log.info(f"No google client found.")
            return TaskMetadata(
                name=get_job_name(task=self.operator),
                inputs=[],
                outputs=[],
                run_facets={},
                job_facets={},
            )

    def extract(self, task_instance) -> Optional[TaskMetadata]:
        return self.extract_on_complete(task_instance)


class AlvinBigQueryExtractor(AlvinBigQueryBaseExtractor):
    """
    This class contains logic for Airflow 2.

    Note that on this version, the BigQuery operator class is
    part of this package: apache-airflow-providers-google.

    https://github.com/OpenLineage/OpenLineage/tree/main/integration/airflow#custom-extractors

    The following env var needs to be set in the airflow host environment:
    OPENLINEAGE_EXTRACTOR_BigQueryExecuteQueryOperator
        =alvin_airflow.lineage.extractors.bigquery.AlvinBigQueryExtractor
    """

    @classmethod
    def get_operator_classnames(cls) -> List[str]:
        return ["BigQueryExecuteQueryOperator", "BigQueryInsertJobOperator"]


class AlvinBigQueryLegacyExtractor(AlvinBigQueryBaseExtractor):
    """
    This class contains logic for Airflow 1.

    Note that on this version, the BigQuery operator class part of the
    Airflow contrib google module.

    https://github.com/OpenLineage/OpenLineage/tree/main/integration/airflow#custom-extractors

    The following env var needs to be set in the airflow host environment:
    OPENLINEAGE_EXTRACTOR_BigQueryOperator
        =alvin_airflow.lineage.extractors.bigquery.AlvinBigQueryLegacyExtractor
    """

    @classmethod
    def get_operator_classnames(cls) -> List[str]:
        return ["BigQueryOperator"]
