import os
import gorilla
import logging
from alvin_airflow.patch.config import patch_list, extractor_list, dags_list
import pkg_resources

target_packages = ["apache-airflow"]

log = logging.getLogger(__name__)


class AlvinAirflowPatchManager:
    """
    Class responsible to manage the installation of Alvin
    components in the Airflow host environment.

    It will check the airflow version and install the
    respective patches and environment variables.
    """

    def __init__(self):
        self.host_package_map = dict()

    def _load_host_packages(self):
        """
        Load the target packages from the host environment
        and creates a map with version
        """
        for target_package in target_packages:
            host_package = pkg_resources.get_distribution(target_package)
            if host_package:
                self.host_package_map[target_package] = host_package

    def _get_supported_patches(self):
        """Check patches compatible with the host packages."""
        supported_patches = []
        for patch in patch_list:
            print(f"Looking at patch: {patch}")
            host_package = self.host_package_map.get(patch.package_name)
            print(f"Found this package: {host_package}")
            if host_package.version in patch.supported_versions:
                supported_patches.append(patch)
                print(f"Adding patch: {patch}")
        return supported_patches

    def _install_lineage_extractors(self):
        """Install the env vars compatible with the host packages."""
        if not self.host_package_map:
            self._load_host_packages()
        for extractor in extractor_list:
            host_package = self.host_package_map.get(extractor.package_name)
            if host_package.version in extractor.supported_versions:
                os.environ[extractor.env_name] = extractor.extractor_path

    def _load_patches(self):
        """Load patches compatible with host environment."""
        self._load_host_packages()

        supported_patches = self._get_supported_patches()

        return supported_patches

    def _install_dags(self):
        if not self.host_package_map:
            self._load_host_packages()
        for dag_config in dags_list:
            host_package = self.host_package_map.get(dag_config.package_name)
            if host_package.version in dag_config.supported_versions:
                dag_creation = dag_config.function
                dag_creation()

    def install(self):
        """Install patches and environment variables."""
        supported_patches = self._load_patches()

        log.info('Installing Alvin Extractors....')
        self._install_lineage_extractors()

        log.info('Installing Alvin DAGs....')
        self._install_dags()

        log.info(self.host_package_map)

        log.info('Installing Alvin Patches....')

        log.info(f"Found {len(supported_patches)} patches from manager")

        settings = gorilla.Settings(allow_hit=True)

        for patch_config in supported_patches:
            patch = gorilla.Patch(
                patch_config.destination,
                patch_config.function.__name__,
                patch_config.function,
                settings=settings,
            )
            log.info(patch)
            gorilla.apply(patch)
            log.info(
                f"Patched {patch.destination.__module__} "
                f"{patch.destination.__name__} {patch.name}"
            )
